==> Omni-0.9.1.tar.gz: Omni/UPDF/UPDFDevice.hpp <==
/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _UPDFDevice
#define _UPDFDevice

#include <Device.hpp>
#include <OmniProxy.hpp>

#include <libxml/parser.h>
#include <libxml/tree.h>

extern "C" {
   PSZCRO       getVersion                  ();
   Enumeration *getDeviceEnumeration        (PSZCRO  pszLibraryName,
                                             bool    fBuildOnly);
   Device      *newDeviceW_Advanced         (bool    fAdvanced);
   Device      *newDeviceW_JopProp_Advanced (PSZCRO  pszJobProperties,
                                             bool    fAdvanced);
   void         deleteDevice                (Device *pDevice);
};

struct UPDFDeviceInstance;

class UPDFDevice : public PrintDevice
{
public:
   enum {
      TOP_TO_BOTTOM = 1
   };

                               UPDFDevice               (xmlDocPtr    docDC,
                                                         xmlDocPtr    docUDR,
                                                         PSZCRO       pszDriverName,
                                                         PSZCRO       pszDeviceName,
                                                         PSZCRO       pszShortName,
                                                         PSZCRO       pszJobProperties,
                                                         PSZCRO       pszXMLDeviceName);
                              ~UPDFDevice               ();

#ifdef INCLUDE_JP_UPDF_BOOKLET
   virtual DeviceBooklet      *getDefaultBooklet        ();
#endif
#ifdef INCLUDE_JP_COMMON_COPIES
   virtual DeviceCopies       *getDefaultCopies         ();
#endif
   virtual PSZCRO              getDefaultDitherID       ();
   virtual DeviceForm         *getDefaultForm           ();
#ifdef INCLUDE_JP_UPDF_JOGGING
   virtual DeviceJogging      *getDefaultJogging        ();
#endif
   virtual DeviceMedia        *getDefaultMedia          ();
#ifdef INCLUDE_JP_COMMON_NUP
   virtual DeviceNUp          *getDefaultNUp            ();
#endif
   virtual DeviceOrientation  *getDefaultOrientation    ();
#ifdef INCLUDE_JP_COMMON_OUTPUT_BIN
   virtual DeviceOutputBin    *getDefaultOutputBin      ();
#endif
   virtual DevicePrintMode    *getDefaultPrintMode      ();
   virtual DeviceResolution   *getDefaultResolution     ();
#ifdef INCLUDE_JP_COMMON_SCALING
   virtual DeviceScaling      *getDefaultScaling        ();
#endif
#ifdef INCLUDE_JP_COMMON_SHEET_COLLATE
   virtual DeviceSheetCollate *getDefaultSheetCollate   ();
#endif
#ifdef INCLUDE_JP_COMMON_SIDE
   virtual DeviceSide         *getDefaultSides          ();
#endif
#ifdef INCLUDE_JP_COMMON_STITCHING
   virtual DeviceStitching    *getDefaultStitching      ();
#endif
   virtual DeviceTray         *getDefaultTray           ();
#ifdef INCLUDE_JP_COMMON_TRIMMING
   virtual DeviceTrimming     *getDefaultTrimming       ();
#endif

   virtual DeviceCommand      *getDefaultCommands       ();
   virtual DeviceData         *getDefaultData           ();
   virtual DeviceString       *getDefaultString         ();

   virtual DeviceGamma        *getCurrentGamma          ();

   static UPDFDevice          *isAUPDFDevice            (Device      *pDevice);

   xmlNodePtr                  findDCEntry              (xmlNodePtr   root,
                                                         PSZCRO       pszName,
                                                         bool         fDebugOutput);
   xmlNodePtr                  findUDREntry             (xmlNodePtr   root,
                                                         PSZCRO       pszName,
                                                         bool         fDebugOutput);
   xmlNodePtr                  findLocaleEntry          (xmlNodePtr   root,
                                                         PSZCRO       pszName,
                                                         bool         fDebugOutput);
   xmlNodePtr                  findCSEntry              (xmlNodePtr   root,
                                                         PSZCRO       pszName,
                                                         bool         fDebugOutput);

   xmlNodePtr                  findDCEntryKeyValue      (PSZCRO       pszKey,
                                                         PSZCRO       pszValue,
                                                         bool         fDebugOutput);
   xmlNodePtr                  findUDREntryKeyValue     (PSZCRO       pszKey,
                                                         PSZCRO       pszValue,
                                                         bool         fDebugOutput);
   xmlNodePtr                  findLocaleEntryKeyValue  (PSZCRO       pszKey,
                                                         PSZCRO       pszValue,
                                                         bool         fDebugOutput);
   xmlNodePtr                  findCSEntryKeyValue      (PSZCRO       pszKey,
                                                         PSZCRO       pszValue,
                                                         bool         fDebugOutput);

   static xmlNodePtr           findEntry                (xmlNodePtr   root,
                                                         PSZCRO       pszName,
                                                         bool         fDebugOutput);

   xmlNodePtr                  findEntryKeyValue        (xmlNodePtr   root,
                                                         PSZCRO       pszKey,
                                                         PSZCRO       pszValue,
                                                         bool         fDebugOutput);

   int                         getXVirtualUnits         ();
   int                         getYVirtualUnits         ();

   xmlNodePtr                  getRootDC                ();
   xmlNodePtr                  getRootUDR               ();
   xmlNodePtr                  getRootLocale            ();
   xmlNodePtr                  getRootCS                ();

private:
   void                        commonInit               (xmlDocPtr    docDC,
                                                         xmlDocPtr    docUDR);

   PSZCRO                      getDefaultFeature        (PSZCRO       pszKey);

   bool                        setVirtualUnits          ();

   UPDFDeviceInstance *pInstance_d;

   xmlDocPtr           docDC_d;          // Device Configuration
   xmlNodePtr          nodeRootDC_d;

   xmlDocPtr           docUDR_d;         // Unit Description Reference
   xmlNodePtr          nodeRootUDR_d;

   xmlDocPtr           docLocale_d;      // Locale
   xmlNodePtr          nodeRootLocale_d;

   xmlDocPtr           docCS_d;          // Command Sequences
   xmlNodePtr          nodeRootCS_d;

   int                 iVirtualUnitsX_d;
   int                 iVirtualUnitsY_d;

   PSZRO               pszDriverName_d;
   PSZRO               pszDeviceName_d;
   PSZRO               pszShortName_d;
};

static inline xmlNodePtr
firstNode (xmlNodePtr elm)
{
   if (!elm)
      return 0;

   while (  0 != elm
         && (  XML_COMMENT_NODE == elm->type
            || XML_TEXT_NODE    == elm->type
            )
         )
   {
      // Move to the next node
      elm = elm->next;
   }

   return elm;
}

static inline xmlNodePtr
nextNode (xmlNodePtr elm)
{
   if (!elm)
      return 0;

   do
   {
      // Move to the next node
      elm = elm->next;

   } while (  0 != elm
           && (  XML_COMMENT_NODE == elm->type
              || XML_TEXT_NODE    == elm->type
              )
           );

   return elm;
}

static inline void
printNode (xmlNodePtr elm,
           bool       fDebugOutput)
{
   if (fDebugOutput)
   {
      xmlElemDump (stderr, elm->doc, elm);
   }
}

#define FINDDCENTRY(pUPDFDevice, \
                    elm,         \
                    pszName,     \
                    pszFunction) \
   pUPDFDevice->findDCEntry (elm, pszName, DebugOutput::shouldOutput##pszFunction ())
#define FINDUDRENTRY(pUPDFDevice, \
                     elm,         \
                     pszName,     \
                     pszFunction) \
   pUPDFDevice->findUDREntry (elm, pszName, DebugOutput::shouldOutput##pszFunction ())
#define FINDLOCALEENTRY(pUPDFDevice, \
                        elm,         \
                        pszName,     \
                        pszFunction) \
   pUPDFDevice->findLocaleEntry (elm, pszName, DebugOutput::shouldOutput##pszFunction ())
#define FINDCSENTRY(pUPDFDevice, \
                    elm,         \
                    pszName,     \
                    pszFunction) \
   pUPDFDevice->findCSEntry (elm, pszName, DebugOutput::shouldOutput##pszFunction ())

#define FINDDCENTRYKEYVALUE(pUPDFDevice, \
                            pszKey,      \
                            pszValue,    \
                            pszFunction) \
   pUPDFDevice->findDCEntryKeyValue (pszKey, pszValue, DebugOutput::shouldOutput##pszFunction ())
#define FINDUDRENTRYKEYVALUE(pUPDFDevice, \
                             pszKey,      \
                             pszValue,    \
                             pszFunction) \
   pUPDFDevice->findUDREntryKeyValue (pszKey, pszValue, DebugOutput::shouldOutput##pszFunction ())
#define FINDLOCALEENTRYKEYVALUE(pUPDFDevice, \
                                pszKey,      \
                                pszValue,    \
                                pszFunction) \
   pUPDFDevice->findLocaleEntryKeyValue (pszKey, pszValue, DebugOutput::shouldOutput##pszFunction ())
#define FINDCSENTRYKEYVALUE(pUPDFDevice, \
                            pszKey,      \
                            pszValue,    \
                            pszFunction) \
   pUPDFDevice->findCSEntryKeyValue (pszKey, pszValue, DebugOutput::shouldOutput##pszFunction ())

#endif

==> Omni-0.9.2.tar.gz: Omni/UPDF/UPDFDevice.hpp <==
/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _UPDFDevice
#define _UPDFDevice

#include <Device.hpp>
#include <OmniProxy.hpp>

#include "XMLInterface.hpp"

extern "C" {
   PSZCRO       getVersion                  ();
   Enumeration *getDeviceEnumeration        (PSZCRO  pszLibraryName,
                                             bool    fBuildOnly);
   Device      *newDeviceW_Advanced         (bool    fAdvanced);
   Device      *newDeviceW_JopProp_Advanced (PSZCRO  pszJobProperties,
                                             bool    fAdvanced);
   void         deleteDevice                (Device *pDevice);
};

struct UPDFDeviceInstance;

class UPDFDevice : public PrintDevice
{
public:
                               UPDFDevice               (XmlDocPtr    docDC,
                                                         XmlDocPtr    docUDR,
                                                         PSZCRO       pszDriverName,
                                                         PSZCRO       pszDeviceName,
                                                         PSZCRO       pszShortName,
                                                         PSZCRO       pszJobProperties,
                                                         PSZCRO       pszXMLDeviceName);
                              ~UPDFDevice               ();

#ifdef INCLUDE_JP_UPDF_BOOKLET
   virtual DeviceBooklet      *getDefaultBooklet        ();
#endif
#ifdef INCLUDE_JP_COMMON_COPIES
   virtual DeviceCopies       *getDefaultCopies         ();
#endif
   virtual PSZCRO              getDefaultDitherID       ();
   virtual DeviceForm         *getDefaultForm           ();
#ifdef INCLUDE_JP_UPDF_JOGGING
   virtual DeviceJogging      *getDefaultJogging        ();
#endif
   virtual DeviceMedia        *getDefaultMedia          ();
#ifdef INCLUDE_JP_COMMON_NUP
   virtual DeviceNUp          *getDefaultNUp            ();
#endif
   virtual DeviceOrientation  *getDefaultOrientation    ();
#ifdef INCLUDE_JP_COMMON_OUTPUT_BIN
   virtual DeviceOutputBin    *getDefaultOutputBin      ();
#endif
   virtual DevicePrintMode    *getDefaultPrintMode      ();
   virtual DeviceResolution   *getDefaultResolution     ();
#ifdef INCLUDE_JP_COMMON_SCALING
   virtual DeviceScaling      *getDefaultScaling        ();
#endif
#ifdef INCLUDE_JP_COMMON_SHEET_COLLATE
   virtual DeviceSheetCollate *getDefaultSheetCollate   ();
#endif
#ifdef INCLUDE_JP_COMMON_SIDE
   virtual DeviceSide         *getDefaultSide           ();
#endif
#ifdef INCLUDE_JP_COMMON_STITCHING
   virtual DeviceStitching    *getDefaultStitching      ();
#endif
   virtual DeviceTray         *getDefaultTray           ();
#ifdef INCLUDE_JP_COMMON_TRIMMING
   virtual DeviceTrimming     *getDefaultTrimming       ();
#endif

   virtual DeviceCommand      *getDefaultCommands       ();
   virtual DeviceData         *getDefaultData           ();
   virtual DeviceString       *getDefaultString         ();

   virtual DeviceGamma        *getCurrentGamma          ();

   static UPDFDevice          *isAUPDFDevice            (Device      *pDevice);

   XmlNodePtr                  findDCEntry              (XmlNodePtr   root,
                                                         PSZCRO       pszName,
                                                         bool         fDebugOutput);
   XmlNodePtr                  findUDREntry             (XmlNodePtr   root,
                                                         PSZCRO       pszName,
                                                         bool         fDebugOutput);
   XmlNodePtr                  findLocaleEntry          (XmlNodePtr   root,
                                                         PSZCRO       pszName,
                                                         bool         fDebugOutput);
   XmlNodePtr                  findCSEntry              (XmlNodePtr   root,
                                                         PSZCRO       pszName,
                                                         bool         fDebugOutput);

   XmlNodePtr                  findDCEntryKeyValue      (PSZCRO       pszKey,
                                                         PSZCRO       pszValue,
                                                         bool         fDebugOutput);
   XmlNodePtr                  findUDREntryKeyValue     (PSZCRO       pszKey,
                                                         PSZCRO       pszValue,
                                                         bool         fDebugOutput);
   XmlNodePtr                  findLocaleEntryKeyValue  (PSZCRO       pszKey,
                                                         PSZCRO       pszValue,
                                                         bool         fDebugOutput);
   XmlNodePtr                  findCSEntryKeyValue      (PSZCRO       pszKey,
                                                         PSZCRO       pszValue,
                                                         bool         fDebugOutput);

   static XmlNodePtr           findEntry                (XmlNodePtr   root,
                                                         PSZCRO       pszName,
                                                         bool         fDebugOutput);

   XmlNodePtr                  findEntryKeyValue        (XmlNodePtr   root,
                                                         PSZCRO       pszKey,
                                                         PSZCRO       pszValue,
                                                         bool         fDebugOutput);

   int                         getXVirtualUnits         ();
   int                         getYVirtualUnits         ();

   XmlNodePtr                  getRootDC                ();
   XmlNodePtr                  getRootUDR               ();
   XmlNodePtr                  getRootLocale            ();
   XmlNodePtr                  getRootCS                ();

private:
   void                        commonInit               (XmlDocPtr    docDC,
                                                         XmlDocPtr    docUDR);

   PSZCRO                      getDefaultFeature        (PSZCRO       pszKey);

   bool                        setVirtualUnits          ();

   UPDFDeviceInstance *pInstance_d;

   XmlDocPtr           docDC_d;          // Device Configuration
   XmlNodePtr          nodeRootDC_d;

   XmlDocPtr           docUDR_d;         // Unit Description Reference
   XmlNodePtr          nodeRootUDR_d;

   XmlDocPtr           docLocale_d;      // Locale
   XmlNodePtr          nodeRootLocale_d;

   XmlDocPtr           docCS_d;          // Command Sequences
   XmlNodePtr          nodeRootCS_d;

   int                 iVirtualUnitsX_d;
   int                 iVirtualUnitsY_d;

   PSZRO               pszDriverName_d;
   PSZRO               pszDeviceName_d;
   PSZRO               pszShortName_d;
};

#define FINDDCENTRY(pUPDFDevice, \
                    elm,         \
                    pszName,     \
                    pszFunction) \
   pUPDFDevice->findDCEntry (elm, pszName, DebugOutput::shouldOutput##pszFunction ())
#define FINDUDRENTRY(pUPDFDevice, \
                     elm,         \
                     pszName,     \
                     pszFunction) \
   pUPDFDevice->findUDREntry (elm, pszName, DebugOutput::shouldOutput##pszFunction ())
#define FINDLOCALEENTRY(pUPDFDevice, \
                        elm,         \
                        pszName,     \
                        pszFunction) \
   pUPDFDevice->findLocaleEntry (elm, pszName, DebugOutput::shouldOutput##pszFunction ())
#define FINDCSENTRY(pUPDFDevice, \
                    elm,         \
                    pszName,     \
                    pszFunction) \
   pUPDFDevice->findCSEntry (elm, pszName, DebugOutput::shouldOutput##pszFunction ())

#define FINDDCENTRYKEYVALUE(pUPDFDevice, \
                            pszKey,      \
                            pszValue,    \
                            pszFunction) \
   pUPDFDevice->findDCEntryKeyValue (pszKey, pszValue, DebugOutput::shouldOutput##pszFunction ())
#define FINDUDRENTRYKEYVALUE(pUPDFDevice, \
                             pszKey,      \
                             pszValue,    \
                             pszFunction) \
   pUPDFDevice->findUDREntryKeyValue (pszKey, pszValue, DebugOutput::shouldOutput##pszFunction ())
#define FINDLOCALEENTRYKEYVALUE(pUPDFDevice, \
                                pszKey,      \
                                pszValue,    \
                                pszFunction) \
   pUPDFDevice->findLocaleEntryKeyValue (pszKey, pszValue, DebugOutput::shouldOutput##pszFunction ())
#define FINDCSENTRYKEYVALUE(pUPDFDevice, \
                            pszKey,      \
                            pszValue,    \
                            pszFunction) \
   pUPDFDevice->findCSEntryKeyValue (pszKey, pszValue, DebugOutput::shouldOutput##pszFunction ())

#endif
